;nyquist plug-in
;version 4
;type process
;name "Align Tracks Multi"
;author "Steve Daulton"
;mergeclips 1
;restoresplits 0
;copyright "GNU General Public License v2.0 or later"
;release 2.4.2


;control text "Aligns selected tracks to the first selected.\n"
;control threshold "Detection threshold" float "" 0.5 0 1
;control channel "When stereo track" choice "Use Left,Use Right" 0


;; Maximum number of samples to analyze
(setf maxlen 1000000)

; Unique ID for *scratch*
(setf id "AlignTracksMulti200220041120")


(defun startsample (sig)
  ;;; Count number of samples to clapperboard
  (do ((val (snd-fetch sig) (snd-fetch sig))
       (i 0 (1+ i)))
      ((or (not val) (> i maxlen)) nil)
    (when (> (abs val) threshold)
      (return-from startsample i))))


(defun trim (samples)
  ;;; Trim samples from start of track
  (format nil "Trim ~a samples" samples)
  (let ((start (/ samples *sound-srate*))
        (max-end (/ ny:all (* 2 *sound-srate*))))
    (if (arrayp *track*)
        (vector (extract-abs start max-end (aref *track* 0))
                (extract-abs start max-end (aref *track* 1)))
        (extract-abs start max-end *track*))))


(defun pad (samples)
  ;;; Prepend track with silent samples
  (format nil "Pad ~a samples" samples)
  (let ((dur (/ samples *sound-srate*)))
    (multichan-expand #'monopad *track* dur)))


(defun monopad (sig seconds)
  ;;; Pad mono sound "sig" by "seconds" of silence.
  (sim (s-rest 0)
       (at-abs seconds (cue sig))))


(defun main (sig)
  (let ((tracknum (get '*track* 'index))
        (selected (length (get '*selection* 'tracks)))
        count)

    (case tracknum
      (1 (setf count (startsample sig))
         (if count
             ; Convert to time so that mixed sample rates will work.
             (putprop '*scratch* (/ count *sound-srate*) 'id)
             (throw 'err "Threshold detection level was not exceeded."))
         (setf rslt ""))
      (t (setf count (startsample (snd-copy sig)))
         (setf firstcount (get '*scratch* 'id))
         (unless count
            (throw 'err "Threshold detection level was not exceeded."))
          ; Convert firstcount back to samples.
          (setf firstcount (* firstcount *sound-srate*))
          (setf rslt
            (if (> count firstcount)
                (trim (- count firstcount))
                (pad (- firstcount count))))))

    (when (= tracknum selected)
      (remprop '*scratch* 'id))

    rslt))


(catch 'err
        (if (arrayp *track*)
            (main (aref *track* channel))
            (main *track*)))
